@@ -124,7 +124,7 @@ def wrap(x, L):
 def gen_cent(pos, vel, mass, ids, multis, randoms, vdev, deltac, fenv, 
     LRG_design_array, LRG_decorations_array, ELG_design_array, 
     ELG_decorations_array, QSO_design_array, QSO_decorations_array, 
-    rsd, inv_velz2kms, lbox, want_LRG, want_ELG, want_QSO, Nthread):
+    rsd, inv_velz2kms, lbox, want_LRG, want_ELG, want_QSO, Nthread, origin):
     """
     Generate central galaxies in place in memory with a two pass numba parallel implementation. 
     """
@@ -238,7 +238,19 @@ def gen_cent(pos, vel, mass, ids, multis, randoms, vdev, deltac, fenv,
                 lrg_z[j1] = pos[i,2]
                 lrg_vz[j1] = vel[i,2] + alpha_c_L * vdev[i] # velocity bias
                 # rsd only applies to the z direction
-                if rsd:
+                if rsd and origin != None:
+                    nx = lrg_x[j1] - origin[0]
+                    ny = lrg_y[j1] - origin[1]
+                    nz = lrg_z[j1] - origin[2]
+                    inv_norm = 1./np.sqrt(nx*nx + ny*ny + nz*nz)
+                    nx *= inv_norm
+                    ny *= inv_norm
+                    nz *= inv_norm
+                    proj = inv_velz2kms * (lrg_vx[j1]*nx + lrg_vy[j1]*ny + lrg_vz[j1]*nz)
+                    lrg_x[j1] = lrg_x[j1]+proj*nx
+                    lrg_y[j1] = lrg_y[j1]+proj*ny
+                    lrg_z[j1] = lrg_z[j1]+proj*nz
+                elif rsd:
                     lrg_z[j1] = wrap(pos[i,2] + lrg_vz[j1] * inv_velz2kms, lbox)
                 lrg_mass[j1] = mass[i]
                 lrg_id[j1] = ids[i]
@@ -252,7 +264,19 @@ def gen_cent(pos, vel, mass, ids, multis, randoms, vdev, deltac, fenv,
                 elg_z[j2] = pos[i,2]
                 elg_vz[j2] = vel[i,2] + alpha_c_E * vdev[i] # velocity bias
                 # rsd only applies to the z direction
-                if rsd:
+                if rsd and origin != None:
+                    nx = elg_x[j2] - origin[0]
+                    ny = elg_y[j2] - origin[1]
+                    nz = elg_z[j2] - origin[2]
+                    inv_norm = 1./np.sqrt(nx*nx + ny*ny + nz*nz)
+                    nx *= inv_norm
+                    ny *= inv_norm
+                    nz *= inv_norm
+                    proj = inv_velz2kms*(elg_vx[j2]*nx+elg_vy[j2]*ny+elg_vz[j2]*nz)
+                    elg_x[j2] = elg_x[j2]+proj*nx
+                    elg_y[j2] = elg_y[j2]+proj*ny
+                    elg_z[j2] = elg_z[j2]+proj*nz
+                elif rsd:
                     elg_z[j2] = wrap(pos[i,2] + elg_vz[j2] * inv_velz2kms, lbox)
                 elg_mass[j2] = mass[i]
                 elg_id[j2] = ids[i]
@@ -266,7 +290,19 @@ def gen_cent(pos, vel, mass, ids, multis, randoms, vdev, deltac, fenv,
                 qso_z[j3] = pos[i,2]
                 qso_vz[j3] = vel[i,2] + alpha_c_Q * vdev[i] # velocity bias
                 # rsd only applies to the z direction
-                if rsd:
+                if rsd and origin != None:
+                    nx = qso_x[j3] - origin[0]
+                    ny = qso_y[j3] - origin[1]
+                    nz = qso_z[j3] - origin[2]
+                    inv_norm = 1./np.sqrt(nx*nx + ny*ny + nz*nz)
+                    nx *= inv_norm
+                    ny *= inv_norm
+                    nz *= inv_norm
+                    proj = inv_velz2kms*(qso_vx[j3]*nx+qso_vy[j3]*ny+qso_vz[j3]*nz)
+                    qso_x[j3] = qso_x[j3]+proj*nx
+                    qso_y[j3] = qso_y[j3]+proj*ny
+                    qso_z[j3] = qso_z[j3]+proj*nz
+                elif rsd:
                     qso_z[j3] = wrap(pos[i,2] + qso_vz[j3] * inv_velz2kms, lbox)
                 qso_mass[j3] = mass[i]
                 qso_id[j3] = ids[i]
@@ -311,7 +347,7 @@ def gen_sats(ppos, pvel, hvel, hmass, hid, weights, randoms, hdeltac, hfenv,
     enable_ranks, ranks, ranksv, ranksp, ranksr, 
     LRG_design_array, LRG_decorations_array, ELG_design_array, ELG_decorations_array,
     QSO_design_array, QSO_decorations_array,
-    rsd, inv_velz2kms, lbox, Mpart, want_LRG, want_ELG, want_QSO, Nthread):
+    rsd, inv_velz2kms, lbox, Mpart, want_LRG, want_ELG, want_QSO, Nthread, origin):
 
     """
     Generate satellite galaxies in place in memory with a two pass numba parallel implementation. 
@@ -453,7 +489,19 @@ def gen_sats(ppos, pvel, hvel, hmass, hid, weights, randoms, hdeltac, hfenv,
                 lrg_vy[j1] = hvel[i, 1] + alpha_s_L * (pvel[i, 1] - hvel[i, 1]) # velocity bias
                 lrg_z[j1] = ppos[i, 2]
                 lrg_vz[j1] = hvel[i, 2] + alpha_s_L * (pvel[i, 2] - hvel[i, 2]) # velocity bias
-                if rsd:
+                if rsd and origin != None:
+                    nx = lrg_x[j1] - origin[0]
+                    ny = lrg_y[j1] - origin[1]
+                    nz = lrg_z[j1] - origin[2]
+                    inv_norm = 1./np.sqrt(nx*nx + ny*ny + nz*nz)
+                    nx *= inv_norm
+                    ny *= inv_norm
+                    nz *= inv_norm
+                    proj = inv_velz2kms*(lrg_vx[j1]*nx+lrg_vy[j1]*ny+lrg_vz[j1]*nz)
+                    lrg_x[j1] = lrg_x[j1]+proj*nx
+                    lrg_y[j1] = lrg_y[j1]+proj*ny
+                    lrg_z[j1] = lrg_z[j1]+proj*nz
+                elif rsd:
                     lrg_z[j1] = wrap(lrg_z[j1] + lrg_vz[j1] * inv_velz2kms, lbox)
                 lrg_mass[j1] = hmass[i]
                 lrg_id[j1] = hid[i]
@@ -465,7 +513,19 @@ def gen_sats(ppos, pvel, hvel, hmass, hid, weights, randoms, hdeltac, hfenv,
                 elg_vy[j2] = hvel[i, 1] + alpha_s_E * (pvel[i, 1] - hvel[i, 1]) # velocity bias
                 elg_z[j2] = ppos[i, 2]
                 elg_vz[j2] = hvel[i, 2] + alpha_s_E * (pvel[i, 2] - hvel[i, 2]) # velocity bias
-                if rsd:
+                if rsd and origin != None:
+                    nx = elg_x[j2] - origin[0]
+                    ny = elg_y[j2] - origin[1]
+                    nz = elg_z[j2] - origin[2]
+                    inv_norm = 1./np.sqrt(nx*nx + ny*ny + nz*nz)
+                    nx *= inv_norm
+                    ny *= inv_norm
+                    nz *= inv_norm
+                    proj = inv_velz2kms*(elg_vx[j2]*nx+elg_vy[j2]*ny+elg_vz[j2]*nz)
+                    elg_x[j2] = elg_x[j2]+proj*nx
+                    elg_y[j2] = elg_y[j2]+proj*ny
+                    elg_z[j2] = elg_z[j2]+proj*nz
+                elif rsd:
                     elg_z[j2] = wrap(elg_z[j2] + elg_vz[j2] * inv_velz2kms, lbox)
                 elg_mass[j2] = hmass[i]
                 elg_id[j2] = hid[i]
@@ -477,7 +537,19 @@ def gen_sats(ppos, pvel, hvel, hmass, hid, weights, randoms, hdeltac, hfenv,
                 qso_vy[j3] = hvel[i, 1] + alpha_s_Q * (pvel[i, 1] - hvel[i, 1]) # velocity bias
                 qso_z[j3] = ppos[i, 2]
                 qso_vz[j3] = hvel[i, 2] + alpha_s_Q * (pvel[i, 2] - hvel[i, 2]) # velocity bias
-                if rsd:
+                if rsd and origin != None:
+                    nx = qso_x[j3] - origin[0]
+                    ny = qso_y[j3] - origin[1]
+                    nz = qso_z[j3] - origin[2]
+                    inv_norm = 1./np.sqrt(nx*nx + ny*ny + nz*nz)
+                    nx *= inv_norm
+                    ny *= inv_norm
+                    nz *= inv_norm
+                    proj = inv_velz2kms*(qso_vx[j3]*nx+qso_vy[j3]*ny+qso_vz[j3]*nz)
+                    qso_x[j3] = qso_x[j3]+proj*nx
+                    qso_y[j3] = qso_y[j3]+proj*ny
+                    qso_z[j3] = qso_z[j3]+proj*nz
+                elif rsd:
                     qso_z[j3] = wrap(qso_z[j3] + qso_vz[j3] * inv_velz2kms, lbox)
                 qso_mass[j3] = hmass[i]
                 qso_id[j3] = hid[i]
@@ -692,12 +764,13 @@ def gen_gals(halos_array, subsample, tracers, params, Nthread, enable_ranks, rsd
     velz2kms = params['velz2kms']
     inv_velz2kms = 1/velz2kms
     lbox = params['Lbox']
+    origin = params['origin']
     # for each halo, generate central galaxies and output to file
     LRG_dict_cent, ELG_dict_cent, QSO_dict_cent, ID_dict_cent = \
     gen_cent(halos_array['hpos'], halos_array['hvel'], halos_array['hmass'], halos_array['hid'], halos_array['hmultis'], 
              halos_array['hrandoms'], halos_array['hveldev'], halos_array['hdeltac'], halos_array['hfenv'], 
              LRG_design_array, LRG_decorations_array, ELG_design_array, ELG_decorations_array, QSO_design_array, 
-             QSO_decorations_array, rsd, inv_velz2kms, lbox, want_LRG, want_ELG, want_QSO, Nthread)
+             QSO_decorations_array, rsd, inv_velz2kms, lbox, want_LRG, want_ELG, want_QSO, Nthread, origin)
     if verbose:
         print("generating centrals took ", time.time() - start)
 
@@ -709,7 +782,7 @@ def gen_gals(halos_array, subsample, tracers, params, Nthread, enable_ranks, rsd
              enable_ranks, subsample['pranks'], subsample['pranksv'], subsample['pranksp'], subsample['pranksr'],
              LRG_design_array, LRG_decorations_array, ELG_design_array, ELG_decorations_array,
              QSO_design_array, QSO_decorations_array, rsd, inv_velz2kms, lbox, params['Mpart'],
-             want_LRG, want_ELG, want_QSO, Nthread)
+             want_LRG, want_ELG, want_QSO, Nthread, origin)
     if verbose:
         print("generating satellites took ", time.time() - start)
 
@@ -735,7 +808,7 @@ def gen_gals(halos_array, subsample, tracers, params, Nthread, enable_ranks, rsd
 
 
 def gen_gal_cat(halo_data, particle_data, tracers, params, Nthread = 16, 
-    enable_ranks = False, rsd = True, write_to_disk = False, savedir = "./", verbose = False):
+    enable_ranks = False, rsd = True, write_to_disk = False, savedir = "./", verbose = False, fn_ext = None):
     """
     pass on inputs to the gen_gals function and takes care of I/O
 
@@ -769,6 +842,9 @@ def gen_gal_cat(halo_data, particle_data, tracers, params, Nthread = 16,
     params : dict
         Dictionary of various simulation parameters. 
 
+    fn_ext: str
+        filename extension for saved files. Only relevant when ``write_to_disk = True``.
+
     Output
     ------
 
@@ -799,8 +875,11 @@ def gen_gal_cat(halo_data, particle_data, tracers, params, Nthread = 16,
                 rsd_string = "_rsd"
             else:
                 rsd_string = ""
-
-            outdir = (savedir) / ("galaxies"+rsd_string)
+            
+            if fn_ext is None:
+                outdir = (savedir) / ("galaxies"+rsd_string)
+            else:
+                outdir = (savedir) / ("galaxies"+rsd_string+fn_ext)
 
             # create directories if not existing
             os.makedirs(outdir, exist_ok = True)
